//---------------------------------------------------------------------------
//
// IniFile.cpp
//
// SUBSYSTEM:   Hook system
//				
// MODULE:      Hook tool    
//				
// DESCRIPTION: Common utilities. 
//              Provides implementation for retrieving data from INI file
// 
//             
// AUTHOR:		Ivo Ivanov (ivopi@hotmail.com)
// DATE:		2001 December v1.00
//
//---------------------------------------------------------------------------

#include "..\Common\Common.h"
#include "..\Common\SysUtils.h"
#include "IniFile.h"

//---------------------------------------------------------------------------
//
// class CIniFile   
//
//---------------------------------------------------------------------------
CIniFile::CIniFile(char* pszFileName)
{
	strcpy(m_szFileName, pszFileName);
}

CIniFile::~CIniFile()
{

}

//
// Retrieve a string value from an INI file
//
void CIniFile::ReadString(
	const char* pszSection, 
	const char* pszIdent, 
	const char* pszDefault,
	char*       pszResult
	)
{
    DWORD dwResult = ::GetPrivateProfileString(
		pszSection,        // section name
		pszIdent,          // key name
		NULL,              // default string
		pszResult,         // destination buffer  
		MAX_PATH,          // size of destination buffer
		m_szFileName       // initialization file name
		);

	if (!dwResult)
		strcpy(pszResult, pszDefault);
}

//
// Retrieve a boolean value from an INI file
//
BOOL CIniFile::ReadBool(
	const char* pszSection, 
	const char* pszIdent, 
	BOOL        bDefault
	)
{
	char    szResult[MAX_PATH];
	BOOL    bResult   = bDefault;
	char    szDefault[MAX_PATH];
		
	BoolToStr(bDefault, szDefault);

	ReadString(
		pszSection, 
		pszIdent,
		szDefault,
		szResult
		);

	bResult = StrToBool(szResult);

	return bResult;
}


//
// Retrieve a Integer value from an INI file
//
int CIniFile::ReadInt(
	const char* pszSection, 
	const char* pszIdent, 
	int         nDefault
	)
{
	char    szResult[MAX_PATH];
	int     nResult   = nDefault;
	char    szDefault[MAX_PATH];
		
	IntToStr(nResult, szDefault);

	ReadString(
		pszSection, 
		pszIdent,
		szDefault,
		szResult
		);

	nResult = atoi(szResult);

	return nResult;
}


BOOL CIniFile::SaveString(
		const char* pszSection, 
		const char* pszIdent, 
		const char* pszSave)
{
	BOOL bResult = ::WritePrivateProfileString(
		pszSection,        // section name
		pszIdent,          // key name
		pszSave,          // destination buffer  
		m_szFileName       // initialization file name
		);

	return bResult;
}



BOOL CIniFile::SaveBool(
		const char* pszSection, 
		const char* pszIdent, 
		const BOOL  bSave)
{
	char    szResult[MAX_PATH];
	BoolToStr(bSave, szResult);

	BOOL bResult = ::WritePrivateProfileString(
		pszSection,        // section name
		pszIdent,          // key name
		szResult,          // destination buffer  
		m_szFileName       // initialization file name
		);
	
	return bResult;
}



BOOL CIniFile::SaveInt(
		const char* pszSection, 
		const char* pszIdent, 
		const int   nSave)
{
	char    szResult[MAX_PATH];
	sprintf(szResult, "%d", nSave);

	BOOL bResult = ::WritePrivateProfileString(
		pszSection,        // section name
		pszIdent,          // key name
		szResult,          // destination buffer  
		m_szFileName       // initialization file name
		);

	return bResult;
}
//--------------------- End of the file -------------------------------------

